# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.dispatch as dispatch
from django.utils.translation import ugettext_lazy as _

""" Cosinnus:Notifications configuration file. 
    See http://git.sinnwerkstatt.com/cosinnus/cosinnus-core/wikis/cosinnus-notifications-guidelines.
"""

""" Signal definitions """
note_comment_posted_on_any = dispatch.Signal(providing_args=["user", "obj", "audience"])
note_comment_posted = dispatch.Signal(providing_args=["user", "obj", "audience"])
note_comment_posted_on_commented_post = dispatch.Signal(providing_args=["user", "obj", "audience"])
note_created = dispatch.Signal(providing_args=["user", "obj", "audience"])


""" Notification definitions.
    These will be picked up by cosinnus_notfications automatically, as long as the 
    variable 'notifications' is present in the module '<app_name>/cosinnus_notifications.py'.
    
    Both the mail and subject template will be provided with the following context items:
        :receiver django.auth.User who receives the notification mail
        :sender django.auth.User whose action caused the notification to trigger
        :receiver_name Convenience, full name of the receiver
        :sender_name Convenience, full name of the sender
        :object The object that was created/changed/modified and which the notification is about.
        :object_url The url of the object, if defined by get_absolute_url()
        :object_name The title of the object (only available if it is a BaseTaggableObject)
        :group_name The name of the group the object is housed in (only available if it is a BaseTaggableObject)
        :site_name Current django site's name
        :domain_url The complete base domain needed to prefix URLs. (eg: 'http://sinnwerkstatt.com')
        :notification_settings_url The URL to the cosinnus notification settings page.
        :site Current django site
        :protocol Current portocol, 'http' or 'https'
        
    
""" 
notifications = {
    'note_comment_posted_on_any': {
        'label': _('A user commented on a news post'), 
        'mail_template': 'cosinnus_note/notifications/note_comment_posted.html',
        'subject_template': 'cosinnus_note/notifications/note_comment_posted_subject.txt',
        'signals': [note_comment_posted_on_any],
        'default': False,
        
        'is_html': True,
        'snippet_type': 'news',
        'event_text': _('%(sender_name)s commented on a news post'),
        'notification_text': _('%(sender_name)s commented on a news post'),
        'subject_text': _('%(sender_name)s commented on a news post'),
        'sub_event_text': _('%(sender_name)s'),
        'data_attributes': {
            'object_name': 'note.get_readable_title', 
            'object_url': 'get_absolute_url', 
            'image_url': 'note.creator.cosinnus_profile.get_avatar_thumbnail_url', # note: receiver avatar, not creator's!
            # no event in comment meta for now. looks ugly
            #'sub_event_meta': 'created_on', # created date
            'sub_image_url': 'creator.cosinnus_profile.get_avatar_thumbnail_url', # the comment creators
            'sub_object_text': 'text',
        },
    },  
    'note_comment_posted': {
        'label': _('A user commented on one of your news posts'), 
        'mail_template': 'cosinnus_note/notifications/note_comment_posted.html',
        'subject_template': 'cosinnus_note/notifications/note_comment_posted_subject.txt',
        'signals': [note_comment_posted],
        'default': True,
        
        'is_html': True,
        'snippet_type': 'news',
        'event_text': _('%(sender_name)s commented on your news post'),
        'notification_text': _('%(sender_name)s commented on one of your news posts'),
        'subject_text': _('%(sender_name)s commented on one of your news posts'),
        'sub_event_text': _('%(sender_name)s'),
        'data_attributes': {
            'object_name': 'note.get_readable_title', 
            'object_url': 'get_absolute_url', 
            'image_url': 'note.creator.cosinnus_profile.get_avatar_thumbnail_url', # note: receiver avatar, not creator's!
            # no event in comment meta for now. looks ugly
            #'sub_event_meta': 'created_on', # created date
            'sub_image_url': 'creator.cosinnus_profile.get_avatar_thumbnail_url', # the comment creators
            'sub_object_text': 'text',
        },
    },  
    'note_comment_posted_in_commented_post': {
        'label': _('A user commented on a news posts you commented in'), 
        'mail_template': 'cosinnus_note/notifications/note_comment_posted_on_commented_post.html',
        'subject_template': 'cosinnus_note/notifications/note_comment_posted_on_commented_post_subject.txt',
        'signals': [note_comment_posted_on_commented_post],
        'default': True,
        
        'is_html': True,
        'snippet_type': 'news',
        'event_text': _('%(sender_name)s commented on a news post you commented in'),
        'subject_text': _('%(sender_name)s commented on a news post you commented in'),
        'sub_event_text': _('%(sender_name)s'),
        'data_attributes': {
            'object_name': 'note.get_readable_title', 
            'object_url': 'get_absolute_url', 
            'image_url': 'note.creator.cosinnus_profile.get_avatar_thumbnail_url', # note: receiver avatar, not creator's!
            # no event in comment meta for now. looks ugly
            #'sub_event_meta': 'created_on', # created date
            'sub_image_url': 'creator.cosinnus_profile.get_avatar_thumbnail_url', # the comment creators
            'sub_object_text': 'text',
        },
    },  
    'note_created': {
        'label': _('A user created a news post'), 
        'mail_template': 'cosinnus_note/notifications/note_created.txt',
        'subject_template': 'cosinnus_note/notifications/note_created_subject.txt',
        'signals': [note_created],
        'default': True,
        
        'is_html': True,
        'snippet_type': 'news',
        'event_text': _('New news post by %(sender_name)s'),
        'notification_text': _('%(sender_name)s created a new news post'),
        'subject_text': _('%(sender_name)s posted in %(team_name)s:'),
        'data_attributes': {
            'object_name': 'get_readable_title', 
            'object_url': 'get_absolute_url', 
            'object_text': 'text',
        },
    },  
}



# -*- coding: utf-8 -*-
from django.contrib import admin

from embed_video.admin import AdminVideoMixin

from cosinnus_note.models import Note


class NoteModelAdmin(AdminVideoMixin, admin.ModelAdmin):
    list_display = ('title', 'group', 'creator',)
    list_filter = ('group',)

admin.site.register(Note, NoteModelAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}
TIME_ZONE = 'Europe/Berlin'
LANGUAGE_CODE = 'en-us'
USE_I18N = True
USE_L10N = True
USE_TZ = True
MEDIA_URL = '/media/'
STATIC_URL = '/static/'
SECRET_KEY = 'test-secret-key'
ROOT_URLCONF = 'tests.urls'
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'taggit',
    'cosinnus',
    'cosinnus_note',
    'tests',
)
if django.VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import embed_video.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('cosinnus', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('text', models.TextField(verbose_name='Text')),
            ],
            options={
                'ordering': ['created_on'],
                'verbose_name': 'Comment',
                'verbose_name_plural': 'Comments',
            },
        ),
        migrations.CreateModel(
            name='Note',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('slug', models.SlugField(max_length=55, blank=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('text', models.TextField(verbose_name='Text')),
                ('video', embed_video.fields.EmbedVideoField(null=True, blank=True)),
                ('attached_objects', models.ManyToManyField(to='cosinnus.AttachedObject', null=True, blank=True)),
                ('creator', models.ForeignKey(related_name='cosinnus_note_note_set', verbose_name='Creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('group', models.ForeignKey(related_name='cosinnus_note_note_set', verbose_name='Group', to='cosinnus.CosinnusGroup')),
            ],
            options={
                'ordering': ['-created', 'title'],
                'abstract': False,
                'verbose_name': 'Note Item',
                'verbose_name_plural': 'Note Items',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from embed_video.fields import EmbedVideoField

from cosinnus.models.tagged import BaseTaggableObjectModel
from django.utils.functional import cached_property
from cosinnus.utils.permissions import filter_tagged_object_queryset_for_user,\
    check_object_read_access
from cosinnus.utils.urls import group_aware_reverse
from cosinnus_note import cosinnus_notifications
from django.contrib.auth import get_user_model

import logging
from django.template.defaultfilters import truncatechars
from cosinnus.models.group import CosinnusPortal
from cosinnus.views.mixins.reflected_objects import MixReflectedObjectsMixin
logger = logging.getLogger('cosinnus')

FACEBOOK_POST_URL = 'https://www.facebook.com/%s/posts/%s' # %s, %s :  user_id, post_id


class Note(BaseTaggableObjectModel):
    
    EMPTY_TITLE_PLACEHOLDER = '---'
    
    SORT_FIELDS_ALIASES = [
        ('title', 'title'), ('creator', 'creator'), ('created', 'created'),
    ]

    text = models.TextField(_('Text'))
    video = EmbedVideoField(blank=True, null=True)
    facebook_post_id = models.CharField(_('Facebook Share'), max_length=255, null=True, blank=True)

    class Meta(BaseTaggableObjectModel.Meta):
        ordering = ['-created', 'title']
        verbose_name = _('Note Item')
        verbose_name_plural = _('Note Items')

    def __init__(self, *args, **kwargs):
        super(Note, self).__init__(*args, **kwargs)
        self._meta.get_field('creator').verbose_name = _('Author')

    def save(self, *args, **kwargs):
        created = bool(self.pk) == False
        
        # take the first youtube url from the textand save it as a video link
        self.video = None
        for url in self.urls:
            if 'youtube.com' in url or 'youtu.be' in url:
                self.video = url
                break
            
        super(Note, self).save(*args, **kwargs)
        if created:
            # todo was created
            cosinnus_notifications.note_created.send(sender=self, user=self.creator, obj=self, audience=get_user_model().objects.filter(id__in=self.group.members).exclude(id=self.creator.pk))

    def get_absolute_url(self):
        kwargs = {'group': self.group, 'slug': self.slug}
        return group_aware_reverse('cosinnus:note:note', kwargs=kwargs)
    
    def get_facebook_post_url(self):
        """ If this post has been posted to facebook and its id is known, returns the URL to the facebook post. 
            @return: A string URL to a facebook post or None """
        if self.facebook_post_id:
            try:
                return FACEBOOK_POST_URL % tuple(self.facebook_post_id.split('_'))
            except:
                if settings.DEBUG:
                    raise
        return None
    
    def get_readable_title(self):
        """ Returns either the title if set, or the text of this news post """
        return self.title if not self.title == self.EMPTY_TITLE_PLACEHOLDER else truncatechars(self.text, 40)
    
    @classmethod
    def get_current(self, group, user):
        """ Returns a queryset of the current upcoming events """
        qs = Note.objects.filter(group=group)
        # mix in reflected objects
        if "%s.%s" % (self._meta.app_label, self._meta.model_name) in settings.COSINNUS_REFLECTABLE_OBJECTS:
            mixin = MixReflectedObjectsMixin()
            qs = mixin.mix_queryset(qs, self._meta.model, group)
        if user:
            qs = filter_tagged_object_queryset_for_user(qs, user)
        return qs
    
    @classmethod
    def get_current_for_portal(self):
        """ Returns a queryset of the current Notes for this portal """
        qs = Note.objects.filter(group__portal=CosinnusPortal.get_current())
        return qs
    
    @cached_property
    def video_id(self):
        """ Returns the video id from a URL as such:
        http://www.youtube.com/watch?v=CENF14Iloxw&hq=1
        """
        if self.video:
            match = re.search(r'[?&]v=([a-zA-Z0-9-_]+)(&|$)', self.video)
            if not match:
                match = re.search(r'youtu.be/([a-zA-Z0-9-_]+)(&|$)', self.video)
            if match:
                vid = match.groups()[0]
                return vid
        return None

    @property
    def video_thumbnail(self):
        vid = self.video_id
        ret = None
        if vid:
            ret = 'http://img.youtube.com/vi/%s/hqdefault.jpg' % vid
        return ret
    
    @property
    def urls(self):
        """ Returns a list of all URLs contained in the note's text """
        return re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', self.text)
    

@python_2_unicode_compatible
class Comment(models.Model):
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('Creator'), on_delete=models.PROTECT)
    created_on = models.DateTimeField(_('Created'), auto_now_add=True, editable=False)
    note = models.ForeignKey(Note, related_name='comments')
    text = models.TextField(_('Text'))

    class Meta:
        ordering = ['created_on']
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')

    def __str__(self):
        return 'Comment on “%(note)s” by %(creator)s' % {
            'note': self.note.title,
            'creator': self.creator.get_full_name(),
        }

    def get_absolute_url(self):
        if self.pk:
            return '%s#comment-%d' % (self.note.get_absolute_url(), self.pk)
        return self.note.get_absolute_url()
    
    def save(self, *args, **kwargs):
        created = bool(self.pk) == False
        super(Comment, self).save(*args, **kwargs)
        if created:
            if not self.note.creator == self.creator:
                # comment was created in own post
                cosinnus_notifications.note_comment_posted.send(sender=self, user=self.creator, obj=self, audience=[self.note.creator])
            # comment was created, for other commenters posts (we skip the post creator because the previous notification precedes)
            try:
                # notifications for people who also commented on the note of this comment
                commenter_ids = list(set(self.note.comments.exclude(creator__id__in=[self.creator_id, self.note.creator_id]).values_list('creator', flat=True)))
                commenters = get_user_model().objects.filter(id__in=commenter_ids)
                cosinnus_notifications.note_comment_posted_on_commented_post.send(sender=self, user=self.creator, obj=self, audience=commenters)
                
                # notification for any members in this group, excepting thos who already commented on the note (because they already received a notification)
                group_members_without_commenters_ids = [member_id for member_id in self.note.group.members if member_id not in [self.creator_id, self.note.creator_id] + commenter_ids]
                all_members = get_user_model().objects.filter(id__in=group_members_without_commenters_ids)
                cosinnus_notifications.note_comment_posted_on_any.send(sender=self, user=self.creator, obj=self, audience=all_members)
            except Exception, e:
                logger.error('There was an error in the note_comments_commented notification. See extra', extra={'exc': e})
                
    @property
    def group(self):
        """ Needed by the notifications system """
        return self.note.group
    
    def grant_extra_read_permissions(self, user):
        """ Comments inherit their visibility from their commented on parent """
        return check_object_read_access(self.note, user)

import django
if django.VERSION[:2] < (1, 7):
    from cosinnus_note import cosinnus_app
    cosinnus_app.register()



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class CosinnusNoteAppConfig(AppConfig):

    name = 'cosinnus_note'
    verbose_name = 'Cosinnus Note'

    def ready(self):
        from cosinnus_note import cosinnus_app
        cosinnus_app.register()




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .base import *


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgres_psycopg2',
        'NAME': 'cosinnus',
        'OPTIONS': {
            'autocommit': True,
        }
    }
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


VERSION = (0, 6, 0, 'alpha', 0)

default_app_config = 'cosinnus_note.apps.CosinnusNoteAppConfig'


def get_version(full=True):
    """Derives a PEP386-compliant version number from VERSION.

    Taken from django.utils.version.get_git_changeset at rev ce5e09353d
    """

    def get_git_changeset():
        """Returns a numeric identifier of the latest git changeset.

        Taken from django.utils.version.get_git_changeset at rev ce5e09353d.
        """
        import datetime
        import subprocess
        from os.path import abspath, dirname
        repo_dir = dirname(dirname(abspath(__file__)))
        git_log = subprocess.Popen(
            'git log --pretty=format:%ct --quiet -1 HEAD',
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            shell=True, cwd=repo_dir, universal_newlines=True)
        timestamp = git_log.communicate()[0]
        try:
            timestamp = datetime.datetime.utcfromtimestamp(int(timestamp))
        except ValueError:
            return None
        return timestamp.strftime('%Y%m%d%H%M%S')

    assert len(VERSION) == 5
    assert VERSION[3] in ('alpha', 'beta', 'rc', 'final')

    # Now build the two parts of the version number:
    # main = X.Y[.Z]
    # sub = .devN - for pre-alpha releases
    #     | {a|b|c}N - for alpha, beta and rc releases

    parts = 2 if VERSION[2] == 0 else 3
    if not full:
        return '.'.join(str(x) for x in VERSION[:2])
    main = '.'.join(str(x) for x in VERSION[:parts])

    sub = ''
    if VERSION[3] == 'alpha' and VERSION[4] == 0:
        git_changeset = get_git_changeset()
        if git_changeset:
            sub = '.dev%s' % git_changeset

    elif VERSION[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[VERSION[3]] + str(VERSION[4])

    return str(main + sub)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import RedirectView
from django.views.generic.detail import DetailView, SingleObjectMixin
from django.views.generic.edit import CreateView, DeleteView, UpdateView
from django.views.generic.list import ListView

from cosinnus.views.attached_object import (CreateViewAttachable,
    UpdateViewAttachable)

from cosinnus.views.mixins.group import (RequireReadMixin, RequireWriteMixin,
    FilterGroupMixin, GroupFormKwargsMixin, DipatchGroupURLMixin)
from cosinnus.views.mixins.user import UserFormKwargsMixin

from cosinnus_note.forms import CommentForm, NoteForm
from cosinnus_note.models import Note, Comment
from django.contrib import messages
from cosinnus.views.mixins.filters import CosinnusFilterMixin
from cosinnus_note.filters import NoteFilter
from cosinnus_note import cosinnus_notifications
from cosinnus.utils.urls import group_aware_reverse, safe_redirect
from cosinnus.utils.pagination import PaginationTemplateMixin
from cosinnus.views.facebook_integration import FacebookIntegrationViewMixin
from django.utils.encoding import force_text
from cosinnus.models.tagged import BaseTagObject
from cosinnus.models.group import CosinnusPortal
from cosinnus.views.mixins.reflected_objects import MixReflectedObjectsMixin,\
    ReflectedObjectRedirectNoticeMixin, ReflectedObjectSelectMixin


class NoteCreateView(FacebookIntegrationViewMixin, RequireWriteMixin, FilterGroupMixin, 
                     GroupFormKwargsMixin, UserFormKwargsMixin, CreateViewAttachable):

    form_class = NoteForm
    model = Note
    template_name = 'cosinnus_note/note_form.html'
    form_view = 'add'
    
    message_success = _('Your news post was added successfully.')
    
    def get_context_data(self, **kwargs):
        context = super(NoteCreateView, self).get_context_data(**kwargs)
        context.update({
            'form_view': self.form_view,
        })
        return context
    
    def form_valid(self, form):
        form.instance.creator = self.request.user
        return super(NoteCreateView, self).form_valid(form)
        
    def form_invalid(self, form):
        """
        If the form is invalid, we simply redirect to the success url, except if we come from the create view.
        """
        return super(NoteCreateView, self).form_invalid(form)

    def post(self, request, *args, **kwargs):
        self.referer = self.request.GET.get('next', request.META.get('HTTP_REFERER', group_aware_reverse('cosinnus:note:list', kwargs={'group':self.group})))
        return super(NoteCreateView, self).post(request, *args, **kwargs)
    
    def get_success_url(self):
        # self.referer is set in post() method
        return safe_redirect(self.referer, self.request)

note_create = NoteCreateView.as_view()


class NoteDeleteView(RequireWriteMixin, FilterGroupMixin, DeleteView):

    model = Note
    template_name_suffix = '_delete'
    
    message_success = _('Your news post was deleted successfully.')
    
    def get_success_url(self):
        # self.referer is set in post() method
        messages.success(self.request, self.message_success)
        return group_aware_reverse('cosinnus:note:list', kwargs={'group':self.group})

note_delete = NoteDeleteView.as_view()


class NoteDetailView(ReflectedObjectRedirectNoticeMixin, ReflectedObjectSelectMixin, 
         RequireReadMixin, FilterGroupMixin, DetailView):

    model = Note
    template_name = 'cosinnus_note/note_detail.html'

note_detail = NoteDetailView.as_view()


class NoteIndexView(RequireReadMixin, RedirectView):

    def get_redirect_url(self, **kwargs):
        return group_aware_reverse('cosinnus:note:list', kwargs={'group': self.group})

note_index = NoteIndexView.as_view()


class NoteListView(RequireReadMixin, CosinnusFilterMixin, MixReflectedObjectsMixin, ReflectedObjectSelectMixin, FilterGroupMixin, 
                   PaginationTemplateMixin, ListView):
    model = Note
    filterset_class = NoteFilter
    per_page = 10
    
    def get_queryset(self, **kwargs):
        qs = super(NoteListView, self).get_queryset()
        qs = qs.prefetch_related('comments__creator__cosinnus_profile', 'attached_objects')
        return qs
    
    def get_context_data(self, **kwargs):
        kwargs.update({
            'form':  NoteForm(group=self.group)
        })
        return super(NoteListView, self).get_context_data(**kwargs)

note_list = NoteListView.as_view()


class NoteEmbedView(DipatchGroupURLMixin, MixReflectedObjectsMixin, FilterGroupMixin, PaginationTemplateMixin, ListView):
    model = Note
    per_page = 10
    template_name = 'cosinnus_note/note_embed.html'
    
    def get_queryset(self, **kwargs):
        """ Only ever show public notes """
        qs = super(NoteEmbedView, self).get_queryset()
        qs = qs.filter(media_tag__visibility=BaseTagObject.VISIBILITY_ALL).prefetch_related('comments__creator__cosinnus_profile', 'attached_objects')
        return qs
    
note_embed = NoteEmbedView.as_view()


class NoteEmbedGlobalView(PaginationTemplateMixin, ListView):
    """ Displays all notes in all Portals in an embeddable view, not just from a specific group """
    
    model = Note
    per_page = 10
    template_name = 'cosinnus_note/note_embed.html'
    
    def get_queryset(self, **kwargs):
        """ Only ever show public notes """
        qs = Note.objects.all()
        qs = qs.filter(media_tag__visibility=BaseTagObject.VISIBILITY_ALL).prefetch_related('comments__creator__cosinnus_profile', 'attached_objects')
        return qs
    
note_embed_global = NoteEmbedGlobalView.as_view()


class NoteEmbedCurrentPortalView(NoteEmbedGlobalView):
    """ Displays all notes in this Portal in an embeddable view, not just from a specific group """
    
    def get_queryset(self, **kwargs):
        """ Only ever show public notes """
        qs = super(NoteEmbedCurrentPortalView, self).get_queryset()
        qs = qs.filter(group__portal=CosinnusPortal.get_current())
        return qs
    
note_embed_current_portal = NoteEmbedCurrentPortalView.as_view()


class NoteUpdateView(RequireWriteMixin, FilterGroupMixin, GroupFormKwargsMixin,
                     UserFormKwargsMixin, UpdateViewAttachable):

    form_class = NoteForm
    model = Note
    template_name = 'cosinnus_note/note_form.html'
    form_view = 'edit'
    
    message_success = _('Your news post was edited successfully.')
    
    def get_context_data(self, **kwargs):
        context = super(NoteUpdateView, self).get_context_data(**kwargs)
        context.update({
            'form_view': self.form_view,
        })
        return context

    def form_valid(self, form):
        messages.success(self.request, self.message_success)
        return super(NoteUpdateView, self).form_valid(form)
    
    def get_success_url(self):
        return group_aware_reverse('cosinnus:note:list', kwargs={'group': self.group})

note_update = NoteUpdateView.as_view()


class CommentCreateView(RequireWriteMixin, FilterGroupMixin, CreateView):

    form_class = CommentForm
    group_field = 'note__group'
    model = Comment
    template_name = 'cosinnus_note/note_detail.html'
    
    message_success = _('Your comment was added successfully.')

    def form_valid(self, form):
        form.instance.creator = self.request.user
        form.instance.note = self.note
        messages.success(self.request, self.message_success)
        return super(CommentCreateView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        context = super(CommentCreateView, self).get_context_data(**kwargs)
        # always overwrite object here, because we actually display the note as object, 
        # and not the comment in whose view we are in when form_invalid comes back
        context.update({
            'note': self.note,
            'object': self.note, 
        })
        return context

    def get(self, request, *args, **kwargs):
        self.note = get_object_or_404(Note, group=self.group, slug=self.kwargs.get('note_slug'))
        return super(CommentCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.note = get_object_or_404(Note, group=self.group, slug=self.kwargs.get('note_slug'))
        self.referer = request.META.get('HTTP_REFERER', self.note.group.get_absolute_url())
        return super(CommentCreateView, self).post(request, *args, **kwargs)
    
    def get_success_url(self):
        # self.referer is set in post() method
        return self.referer

comment_create = CommentCreateView.as_view()


class CommentDeleteView(RequireWriteMixin, FilterGroupMixin, DeleteView):

    group_field = 'note__group'
    model = Comment
    template_name_suffix = '_delete'
    
    message_success = _('Your comment was deleted successfully.')
    
    def get_context_data(self, **kwargs):
        context = super(CommentDeleteView, self).get_context_data(**kwargs)
        context.update({'note': self.object.note})
        return context
    
    def post(self, request, *args, **kwargs):
        self.comment = get_object_or_404(Comment, pk=self.kwargs.get('pk'))
        self.referer = request.META.get('HTTP_REFERER', self.comment.note.group.get_absolute_url())
        return super(CommentDeleteView, self).post(request, *args, **kwargs)

    def get_success_url(self):
        # self.referer is set in post() method
        messages.success(self.request, self.message_success)
        return self.referer

comment_delete = CommentDeleteView.as_view()


class CommentDetailView(SingleObjectMixin, RedirectView):

    model = Comment

    def get(self, request, *args, **kwargs):
        obj = self.get_object()
        return HttpResponseRedirect(obj.get_absolute_url())

comment_detail = CommentDetailView.as_view()


class CommentUpdateView(RequireWriteMixin, FilterGroupMixin, UpdateView):

    form_class = CommentForm
    group_field = 'note__group'
    model = Comment
    template_name_suffix = '_update'

    def get_context_data(self, **kwargs):
        context = super(CommentUpdateView, self).get_context_data(**kwargs)
        context.update({'note': self.object.note})
        return context
    
    def post(self, request, *args, **kwargs):
        self.comment = get_object_or_404(Comment, pk=self.kwargs.get('pk'))
        self.referer = request.META.get('HTTP_REFERER', self.comment.note.group.get_absolute_url())
        return super(CommentUpdateView, self).post(request, *args, **kwargs)

    def get_success_url(self):
        # self.referer is set in post() method
        return self.referer

comment_update = CommentUpdateView.as_view()





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.COSINNUS_TAG_OBJECT_MODEL),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('cosinnus_note', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='note',
            name='media_tag',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to=settings.COSINNUS_TAG_OBJECT_MODEL),
        ),
        migrations.AddField(
            model_name='comment',
            name='creator',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, verbose_name='Creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='comment',
            name='note',
            field=models.ForeignKey(related_name='comments', to='cosinnus_note.Note'),
        ),
        migrations.AlterUniqueTogether(
            name='note',
            unique_together=set([('group', 'slug')]),
        ),
    ]



'''
Created on 05.08.2014

@author: Sascha
'''
from django.utils.translation import ugettext_lazy as _

from cosinnus.views.mixins.filters import CosinnusFilterSet
from cosinnus.forms.filters import AllObjectsFilter, SelectCreatorWidget,\
    DropdownChoiceWidget
from django_filters.filters import ChoiceFilter
from django.core.exceptions import ImproperlyConfigured
from cosinnus_note.models import Note
from django.db.models.aggregates import Count


class NoteFilter(CosinnusFilterSet):
    creator = AllObjectsFilter(label=_('Created By'), widget=SelectCreatorWidget)
    
    class Meta:
        model = Note
        fields = ['creator']
        order_by = (
            ('-created', _('Newest Created')),
            ('-num_comments', _('Popularity')),
        )
        
    @property
    def qs(self):
        if not hasattr(self, '_qs') and hasattr(self, 'queryset'):
            self.queryset = self.queryset.annotate(num_comments=Count('comments'))
        return super(NoteFilter, self).qs
    
    def get_order_by(self, order_value):
        return super(NoteFilter, self).get_order_by(order_value)
    


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import patterns, url

cosinnus_root_patterns = patterns('cosinnus_note.views', 
    url(r'^notes/embed/all/$',
        'note_embed_global',
        name='embed-global'),
                                  
    url(r'^notes/embed/$',
        'note_embed_current_portal',
        name='embed-current-portal'),
)
    
cosinnus_group_patterns = patterns('cosinnus_note.views',
    url(r'^$',
        'note_index',
        name='index'),

    url(r'^list/$',
        'note_list',
        name='list'),
                                   
    url(r'^embed/$',
        'note_embed',
        name='embed'),

    #url(r'^list/(?P<tag>[^/]+)/$',
    #    'note_list',
    #    name='list-filtered'),

    url(r'^add/$',
        'note_create',
        name='add'),

    url(r'^(?P<slug>[^/]+)/$',
        'note_detail',
        name='note'),

    url(r'^(?P<slug>[^/]+)/delete/$',
        'note_delete',
        name='delete'),

    url(r'^(?P<slug>[^/]+)/update/$',
        'note_update',
        name='update'),

    url(r'^(?P<note_slug>[^/]+)/comment/$',
        'comment_create',
        name='comment'),

    url(r'^comment/(?P<pk>\d+)/$',
        'comment_detail',
        name='comment-detail'),

    url(r'^comment/(?P<pk>\d+)/delete/$',
        'comment_delete',
        name='comment-delete'),

    url(r'^comment/(?P<pk>\d+)/update/$',
        'comment_update',
        name='comment-update'),
                                   
)


urlpatterns = cosinnus_group_patterns + cosinnus_root_patterns



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms

from cosinnus.forms.attached_object import FormAttachableMixin
from cosinnus.forms.group import GroupKwargModelFormMixin
from cosinnus.forms.tagged import get_form, BaseTaggableObjectForm
from cosinnus.forms.user import UserKwargModelFormMixin

from cosinnus_note.models import Comment, Note


class _NoteForm(GroupKwargModelFormMixin, UserKwargModelFormMixin,
                FormAttachableMixin, BaseTaggableObjectForm):

    class Meta:
        model = Note
        fields = ('title', 'text', 'video',)
        
    
    def __init__(self, *args, **kwargs):
        super(_NoteForm, self).__init__(*args, **kwargs)
        if 'title' in self.initial and self.initial['title'] == Note.EMPTY_TITLE_PLACEHOLDER:
            self.initial['title'] = ''
        if self.fields['title'].initial == Note.EMPTY_TITLE_PLACEHOLDER:
            self.fields['title'].initial = ''
        
    def clean(self):
        """ Insert a placeholder title if no title is given """
        title = self.cleaned_data.get('title', None)
        if not title:
            note_text = self.cleaned_data.get('text', None)
            if note_text:
                self.cleaned_data.update({'title': Note.EMPTY_TITLE_PLACEHOLDER},)
                self.errors.pop('title', None)
        return super(_NoteForm, self).clean()


#: A django-multiform :class:`MultiModelForm`. Includs support for `group` and
#: `attached_objects_querysets` arguments being passed to the underlying main
#: form (:class:`_NoteForm`)
NoteForm = get_form(_NoteForm)


class CommentForm(forms.ModelForm):

    class Meta:
        model = Comment
        fields = ('text',)



# -*- coding: utf-8 -*-
"""
Created on 08.07.2014

@author: Sascha Narr
"""
from __future__ import unicode_literals

from cosinnus.utils.renderer import BaseRenderer
from cosinnus_note.models import Note


class NoteRenderer(BaseRenderer):
    
    model = Note

    template = 'cosinnus_note/attached_notes.html'
    template_single = 'cosinnus_note/single_note_detailed.html'
    template_list = 'cosinnus_note/note_list_standalone.html'
    

    @classmethod
    def render(cls, context, myobjs):
        return super(NoteRenderer, cls).render(context, notes=myobjs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('cosinnus_note', '0003_auto_20160125_1339'),
    ]

    operations = [
        migrations.AddField(
            model_name='note',
            name='facebook_post_id',
            field=models.CharField(max_length=255, null=True, verbose_name='Facebook Share', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from haystack import indexes

from cosinnus.utils.search import BaseTaggableObjectIndex

from cosinnus_note.models import Note


class NoteIndex(BaseTaggableObjectIndex, indexes.Indexable):

    def get_model(self):
        return Note



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('cosinnus_note', '0002_auto_20151104_1711'),
    ]

    operations = [
        migrations.AlterField(
            model_name='note',
            name='group',
            field=models.ForeignKey(related_name='cosinnus_note_note_set', verbose_name='Group', to=settings.COSINNUS_GROUP_OBJECT_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from cosinnus.utils.dashboard import DashboardWidget, DashboardWidgetForm

from cosinnus_note.models import Note
from cosinnus_note.views import NoteCreateView
from django.core.exceptions import ImproperlyConfigured
from cosinnus_note.forms import NoteForm
from cosinnus.views.mixins.reflected_objects import MixReflectedObjectsMixin,\
    ReflectedObjectSelectMixin



class DetailedNotesForm(DashboardWidgetForm):
    amount = forms.IntegerField(label="Amount", initial=3, min_value=0,
        help_text="0 means unlimited", required=False)


class BaseNotesWidget(MixReflectedObjectsMixin, ReflectedObjectSelectMixin, DashboardWidget):

    app_name = 'note'
    model = Note
    template_name = None
    user_model_attr = None
    widget_template_name = 'cosinnus_note/widgets/base_news_widget.html'

    def get_data(self, offset=0):
        """ Returns a tuple (data, rows_returned, has_more) of the rendered data and how many items were returned.
            if has_more == False, the receiving widget will assume no further data can be loaded.
         """
        count = int(self.config['amount'])
        qs = self.get_queryset().all()
        if count != 0:
            qs = qs[offset:offset+count]

        data = {
            'notes': qs,
            'group': self.config.group,
            'no_data': _('No news'),
            'widget_id': self.id,
            'widget_title': self.title,
        }
        data.update(self.get_reflect_data(self.request, self.config.group))

        return (render_to_string(self.get_template_name(), data,
                                context_instance=RequestContext(self.request)), len(qs), len(qs) >= count)

    def get_template_name(self):
        if self.template_name is None:
            raise ImproperlyConfigured("No template_name given")
        return self.template_name



class DetailedNotes(BaseNotesWidget):
    """ This widget acts as a combined group and user widget.
        Group widget: Contains a note-post form and the latest news
        User widget: Contains the latest news posts from all the user's groups.
            Note!: The user note widget need to get the base_widget.html template instead
                    of the fadedown_base_widget.html template as it contains no form!
     """
    
    form_class = DetailedNotesForm
    template_name = 'cosinnus_note/widgets/detailed_news_content.html'
    title = _('Write a news post')
    widget_name = 'detailed_news_list'
    
    def __init__(self, request, config_instance):
        super(DetailedNotes, self).__init__(request, config_instance)
        # Adjust title for user widget
        if not self.config.group:
            self.title = _('News stream')
            
    def render(self, **kwargs):
        # Only for the group dashboard:
        group = getattr(self.config, 'group', None)
        if group:  
            kwargs.update({
                'form':  NoteForm(group=group)
            })
        
        return super(DetailedNotes, self).render(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from cosinnus.core.registries.attached_objects import attached_object_registry

from cosinnus.conf import settings

def register():
    if 'cosinnus_note' in getattr(settings, 'COSINNUS_DISABLED_COSINNUS_APPS', []):
        return
    
    # Import here to prevent import side effects
    from django.utils.translation import ugettext_lazy as _
    from django.utils.translation import pgettext_lazy

    from cosinnus.core.registries import (app_registry, url_registry,
        widget_registry)

    app_registry.register('cosinnus_note', 'note', _('Notes'), deactivatable=True)
    attached_object_registry.register('cosinnus_note.Note',
                         'cosinnus_note.utils.renderer.NoteRenderer')
    url_registry.register_urlconf('cosinnus_note', 'cosinnus_note.urls')
    widget_registry.register('note', 'cosinnus_note.dashboard.DetailedNotes')

    # makemessages replacement protection
    name = pgettext_lazy("the_app", "note")


#!/usr/bin/env python
import os
import codecs
from setuptools import setup, find_packages


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Compile the list of packages available, because setuptools doesn't have
# an easy way to do this. Taken from Django.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
pkgdir = 'cosinnus_note'

for dirpath, dirnames, filenames in os.walk(pkgdir):
    # Ignore PEP 3147 cache dirs and those whose names start with '.'
    dirnames[:] = [d for d in dirnames if not d.startswith('.') and d != '__pycache__']
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

from cosinnus_note import get_version

setup(
    name='cosinnus-note',
    version=get_version(),
    description='cosinnus note and news post application',
    long_description=read('README'),
    author='Sinnwerkstatt Medienagentur GmbH Berlin',
    author_email='web@sinnwerkstatt.com',
    packages=find_packages(exclude=["tests"]),
    data_files=data_files,
    install_requires=[
        'cosinnus>=0.4.2.dev0',
        'django-extra-views>=0.6.0',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Framework :: Django',
    ],
    zip_safe=False,
    include_package_data=True
)
